# (C) 2005-2009. University of Washington. All rights reserved.

from Products.CMFCore.utils import getToolByName
from Products.CMFEditions.setuphandlers import DEFAULT_POLICIES

def setVersionedTypes(portal):
    """Set up the types to be versioned.

    This is here instead of in teh policy because I want
    the types versioned no matter which policy is used.
    """
    portal_repository = getToolByName(portal,
                                      'portal_repository')
    versionable_types = list(portal_repository.getVersionableContentTypes())

    for type_id in ('collective.types.Citation',):
        if type_id not in versionable_types:
            versionable_types.append(type_id)
            #Add default versioning policies to the versioned type
            for policy_id in DEFAULT_POLICIES:
                portal_repository.addPolicyForContentType(type_id,
                                                          policy_id)
    portal_repository.setVersionableContentTypes(versionable_types)

def importVarious(context):
    portal = context.getSite()
    setVersionedTypes(portal)



# (C) 2005-2009. University of Washington. All rights reserved.

from unittest import TestSuite
from Products.CMFCore.utils import getToolByName

from collective.types.citation.tests.base import CitationTestCase
from collective.types.citation import Citation
from collective.types.citation.content import ICitation

class TestSetupTypes(CitationTestCase):
    """Make sure we can install types, etc.
    """

    def afterSetUp(self):
        self.types = getToolByName(self.portal, 'portal_types')
        #Create a citation test folder
        self.folder.invokeFactory('Folder', id='test_citation')
        self.folder = self.folder.test_citation

    def testSetup(self):
        qi = getToolByName(self.portal, 'portal_quickinstaller')
        self.assertTrue(qi.isProductInstalled('collective.types.citation'),
                        'collective.types.citation installation failed')
        
    def test_types_versioned(self):
        """Make sure the types are versioned.
        I put this here instead of in policy because I want
        the types versioned no matter which policy is used.
        """
        repository = getToolByName(self.portal,
                                   'portal_repository')
        versionable = repository.getVersionableContentTypes()
        self.failUnless('collective.types.Citation' in versionable,
                        'collective.types.Citation is not versionable')

    def test_citation_installed(self):
        self.failUnless('collective.types.Citation' in
                        self.types.objectIds(),
                        'The collective.types.Citation type is not installed')


def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite

    suite = TestSuite()
    suite.addTest(makeSuite(TestSetupTypes))

    return suite




# (C) 2005-2009. University of Washington. All rights reserved. 

from doctest import ELLIPSIS
from doctest import NORMALIZE_WHITESPACE
import unittest
from Testing.ZopeTestCase import FunctionalDocFileSuite as Suite

from collective.types.citation.tests.base import CitationFunctionalTestCase

#Doctests relative to the main project directory
#(must contain file extension)
list_doctests = ['content.py',
                 'tests/citation.txt',]

def test_suite():
    return unittest.TestSuite(
        [Suite(filename,
               optionflags=(NORMALIZE_WHITESPACE | ELLIPSIS),
               package='collective.types.citation',
               test_class=CitationFunctionalTestCase)
         for filename in list_doctests]
        )



# (C) 2005-2009. University of Washington. All rights reserved.

class ViewNotFoundException(Exception): pass

def _getInnerViewName(outer_view):
    """Get the name of the inner view for this object,
    the view name with _page cut off of the end.

    Returns a str
    """
    if '_page' == outer_view[-5:]:
        inner_view = outer_view[:-5]
    else:
        inner_view = outer_view
    view = ''.join(('@@', inner_view))
    return view

def extractInnerView(context):
    """Get the inner view for this context, the view
    of the item without page or Collage wrappers.

    Returns the html of that view.
    """
    view = _getInnerViewName(context.getLayout())
    try:
        return context.restrictedTraverse(view)()
    except Exception, msg1:
        #The view lookup failed, possibly because of the
        #@@ that got added to the beginning.
        #Try again, without the @@.
        #This is required for the built-in types, which
        #are not set up as Zope 3 style views.
        #If this one fails, go ahead and raise the
        #exception, with information on which view
        #failed on which object
        try:
            return context.restrictedTraverse(view[2:])()
        except Exception, msg2:
            raise ViewNotFoundException,\
                  "The view %(view)s could not be "\
                  "fetched on %(item)s (%(type)s) at "\
                  "%(location)s\n"\
                  "%(msg1)s\n%(msg2)s" %\
                  dict(view=view,
                       item=context.title_or_id(),
                       type=context.Type(),
                       location=context.absolute_url(),
                       msg1=msg1,
                       msg2=msg2)



# (C) 2005-2009. University of Washington. All rights reserved.

#Get the classes and such from the content directory
from collective.types.citation.content import *
from collective.types.citation import config

from Products.Archetypes import atapi
from Products.CMFCore import utils
from zope.i18nmessageid import MessageFactory
CitationMessageFactory = MessageFactory('collective.types.citation')

PROJECTNAME = 'collective.types.citation'

def initialize(context):
    """Initializer called when used as a Zope 2 product."""
    content_types, constructors, ftis = atapi.process_types(
        atapi.listTypes(PROJECTNAME),
        PROJECTNAME)

    for atype, constructor in \
            zip(content_types, constructors):
        utils.ContentInit("%s: %s" % (PROJECTNAME,
                                      atype.portal_type),
                          content_types = (atype,),
                          permission =
            config.ADD_PERMISSIONS[atype.portal_type],
                          extra_constructors = (constructor,),
                          ).initialize(context)



# (C) 2005-2009. University of Washington. All rights reserved.

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile \
     import ViewPageTemplateFile
from plone.memoize.instance import memoize

from util import extractInnerView as getView

class ViewPage(BrowserView):
    """View class for a number of types as setup in configure.zcml
    """
    __call__ = ViewPageTemplateFile('page_wrapper.pt')

    def getInnerView(self, template_name=None):
        """Get the view for the context, and return
        the html.

        Returns a string containing html.
        """
        self.context.thisTemplate = template_name
        view = getView(self.context)

        #Reset the template, or it will be stored in memory
        self.context.thisTemplate = ''
        return view




# (C) 2005-2009. University of Washington. All rights reserved.

from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile \
     import ViewPageTemplateFile

from util import extractInnerView as getView

#Patch for a bug that causes these views not to work
from Products.Five.browser.pagetemplatefile import \
     ZopeTwoPageTemplateFile
        
def _getContext(self):
    while 1:
        self = self.aq_parent
        if not getattr(self, '_is_wrapperish', None):
            return self

class CollageView(BrowserView):
    """Grab the selected view, and stick it into the
    Collage layout.
    """
    __call__ = ViewPageTemplateFile('collage.pt')
    
    def __init__(self, context, request):
        self.context = context
        self.request = request

        #Monkey patch for the strange error shown here
        #https://bugs.launchpad.net/zope2/+bug/176566
        ZopeTwoPageTemplateFile._getContext = _getContext

    def getInnerView(self):
        return getView(self.context)



# (C) 2005-2009. University of Washington. All rights reserved.
"""Common configuration constants
"""
from Products.CMFCore.permissions import AddPortalContent

PROJECTNAME = "collective.types.citation"
IMAGE_KEY = 'image'
ADD_PERMISSIONS = {'collective.types.Citation': AddPortalContent,}



# (C) 2005-2009. University of Washington. All rights reserved.

from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile \
     import ViewPageTemplateFile
#from plone.memoize.instance import memoize
from myph.types.interfaces import ICitation

#Patch for a bug that causes these views not to work
from Products.Five.browser.pagetemplatefile import \
     ZopeTwoPageTemplateFile
        
def _getContext(self):
    while 1:
        self = self.aq_parent
        if not getattr(self, '_is_wrapperish', None):
            return self

class CitationView(BrowserView):
    """Parent Citation View
    """
    def __init__(self, context, request):
        self.context = context
        self.request = request

        #Monkey patch for the strange error shown here
        #https://bugs.launchpad.net/zope2/+bug/176566
        ZopeTwoPageTemplateFile._getContext = _getContext

class CitationPageView(CitationView):
    """Article view mode for a Citation
    """
    template = 'citation_page.pt'
    __call__ = ViewPageTemplateFile(template)

    def getLayout(self):
        selected = self.context.getLayout()
        view_name = selected[:-5]
        view = ''.join(('@@', view_name))
        return self.context.restrictedTraverse(view)()

class ArticleView(CitationView):
    """Article view mode for a Citation
    """
    template = 'citation_article.pt'
    __call__ = ViewPageTemplateFile(template)

class BookView(CitationView):
    """Book view mode for a Citation
    """
    __call__ = ViewPageTemplateFile('citation_book.pt')

class ReportView(CitationView):
    """Report view mode for a Citation
    """
    __call__ = ViewPageTemplateFile('citation_report.pt')

class OtherView(CitationView):
    """Default view mode for a Citation
    """
    __call__ = ViewPageTemplateFile('citation_other.pt')



# (C) 2005-2009. University of Washington. All rights reserved.

# Define a common migration base class for use in all
# migration tests

from Products.Five import zcml
from Products.Five import fiveconfigure
from Products.Five.testbrowser import Browser

from Products.PloneTestCase.layer import onsetup

# Import the base test case classes
from Testing import ZopeTestCase as ztc
from Products.PloneTestCase import PloneTestCase as ptc

from Products.PloneTestCase.setup import portal_owner as user
from Products.PloneTestCase.setup import default_password as password

from Products.CMFPlone.tests.testQueryCatalog import AddPortalTopics

# Set up the Plone site used for the test fixture. The PRODUCTS are the products
# to install in the Plone site (as opposed to the products defined above, which
# are all products available to Zope in the test fixture)
PRODUCTS = ['collective.types.citation',]
from OFS.Application import get_folder_permissions, get_products

@onsetup
def setup_citation():
    import collective.types.citation
    fiveconfigure.debug_mode = True
    zcml.load_config('configure.zcml', collective.types.citation)
    zcml.load_site()
    fiveconfigure.debug_mode = False

    ztc.installPackage('collective.types.citation')

setup_citation()
ptc.setupPloneSite(products=PRODUCTS)

class Session(dict):
    def set(self, key, value):
        self[key] = value

class FakeContext(object):
    """Fake context for testing"""
    pass

class CitationTestCase(ptc.PloneTestCase):
    """Base class used for test cases
    """

    def _setup(self):
        ptc.PloneTestCase._setup(self)
        self.app.REQUEST['SESSION'] = Session()

    def afterSetup(self):
        #This little gem actually makes the session info
        #stick, even in doctests
        ztc.utils.setupCoreSessions(self.app)

    def makeBrowser(self):
        browser = Browser()
        browser.mech_browser.request = self.app.REQUEST

        #False shows all errors, True handles some
        browser.handleErrors = False

        return browser

class CitationFunctionalTestCase(ptc.FunctionalTestCase,
                                 ptc.PloneTestCase):
    """Test case class used for functional (doc-)tests
    """
    def _setup(self):
        ptc.PloneTestCase._setup(self)
        self.app.REQUEST['SESSION'] = Session()
        
    def afterSetup(self):
        #This little gem actually makes the session info
        #stick, even in doctests
        ztc.utils.setupCoreSessions(self.app)
        #self.portal.error_log._ignored_exceptions = ()

    def makeBrowser(self):
        browser = Browser()
        browser.mech_browser.request = self.app.REQUEST

        #False shows all errors, True handles some
        browser.handleErrors = False

        return browser

    def makeAdminBrowser(self):
        """Browser that is logged in as admin"""
        browser = Browser()
        browser.mech_browser.request = self.app.REQUEST

        #False shows all errors, True handles some
        browser.handleErrors = False

        #Log in as the admin user
        browser.open(self.folder.absolute_url())
        form = browser.getForm(name='loginform')
        form.getControl(name='__ac_name').value = user
        form.getControl(name='__ac_password').value = password
        form.submit()

        return browser



# (C) 2005-2009. University of Washington. All rights reserved.

from unittest import TestSuite

from collective.types.citation.tests.base import CitationTestCase
from collective.types.citation.tests.base import Session
from collective.types.citation import Citation
from collective.types.citation.content import ICitation

class TestViews(CitationTestCase):
    """Make sure the different views work properly
    """
    def afterSetUp(self):
        self.folder.invokeFactory('collective.types.Citation',
                                  id=u'cite2')

        cite = self.folder.cite2
        cite.setTitle('Citation Title')
        cite.setCite_author('Author')
        cite.setLocation('Location')
        cite.setFullTextLink('http://test.com')
        cite.setSource('Source')
        cite.setVolume('Volume')
        cite.setIssue('Issue')
        cite.setPages('4')
        cite.setAbstractLink('http://Abstract.com')
        cite.REQUEST.SESSION = Session()
        cite.REQUEST.SESSION.id = '123'

    def testChangeLayout(self):
        """Make sure all layouts can be switched to
        """
        cite = self.folder.cite2

        for layout in ('article_page', 'book_page',
                       'report_page', 'other_page'):
            cite.setLayout(layout)
            self.assertEquals(cite.getLayout(),
                              layout,
                              'Layout could not be '\
                              'changed to %s' % layout)
        
    def testLayouts(self):
        """Make sure the layouts don't throw any errors
        """
        cite = self.folder.cite2

        for layout in ('article', 'article_page',
                       'book', 'book_page',
                       'report', 'report_page',
                       'other', 'other_page'):
            found = cite.restrictedTraverse(layout)()
            #self.failUnless('http://example.com' in
            #                found,
            #                '%s layout was not '\
            #                'outputted properly.\n'\
            #                'Found:\n%s' % (layout, found))


class TestCreateCitation(CitationTestCase):
    """Make sure we can create Citations
    """

    def testCreateCitation(self):
        self.folder.invokeFactory('collective.types.Citation',
                                  id='cite1')
        cite = self.folder.cite1
        self.assertEquals('cite1', cite.id,
                          "Citation id should be 'cite1' "\
                          ", instead it's %s" % \
                          cite.id)
        self.assertEquals(cite.default_view, 'base_view',
                          'default_view should be base_view')

def test_suite():
    from unittest import TestSuite
    from unittest import makeSuite

    suite = TestSuite()
    suite.addTest(makeSuite(TestCreateCitation))
    suite.addTest(makeSuite(TestViews))

    return suite




from setuptools import setup, find_packages
import os

version = '9.06'

def read(*pathnames):
    return open(os.path.join(os.path.dirname(__file__), *pathnames)).read()

setup(name='collective.types.citation',
      version=version,
      description="Citation content type",
      long_description=''.join((read('docs', 'README.txt'),
                                read('docs', 'HISTORY.txt'),
                                read('docs', 'INSTALL.txt'),
                                read('docs', 'CUSTOMIZATION.txt'))),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='plone types citation',
      author='Liz Dahlstrom',
      author_email='lhill2@u.washington.edu',
      url='http://cphi.washington.edu',
      license='GPL 2',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective','collective.types'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



# (C) 2005-2009. University of Washington. All rights reserved. 

from Products.ATContentTypes.content import base
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.Archetypes.atapi import AnnotationStorage
from Products.Archetypes.atapi import ATFieldProperty
from Products.Archetypes.atapi import IntegerWidget
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import StringField
from Products.Archetypes.atapi import StringWidget
from Products.CMFPlone import PloneMessageFactory as _

from zope import schema
from zope.app.container.constraints import contains
from zope.interface import implements
from zope.interface import Interface

class ICitation(Interface):
    """A citation from a journal or website

    Note that only the title and description are required,
    the user may have any mix of additional information.
    """
    title = schema.TextLine(
        title=_(u'Title'),
        required=True)
    description = schema.Text(
        title=_(u'Description'),
        required=True)
    cite_author = schema.TextLine(
        title=_(u'Author(s)'),
        description=_(u'The author or authors of the work being cited'),
        required=False)
    year = schema.TextLine(
        title=_(u'Year'),
        description=_(u'The year that the work being cited was published'),
        required=False)
    location = schema.TextLine(
        title=_(u'Location'),
        description=_(u'Publication Location'),
        required=False)
    fulltext_link = schema.URI(
        title=_(u'Full Text URL'),
        description=_(u'Link to the full text of the work being cited'),
        required=False)
    source = schema.TextLine(
        title=_('Source/Journal/Publisher'),
        description=_('Source/Journal'),
        required=False)
    volume = schema.TextLine(
        title=_(u'Volume'),
        description=_(u'Publication Volume'),
        required=False)
    issue = schema.TextLine(
        title=_(u'Issue'),
        description=_(u'Publication Issue'),
        required=False)
    pages = schema.Int(
        title=_(u'Pages'),
        description=_(u'Number of pages'),
        required=False)
    abstract_link = schema.URI(
        title=_(u'Abstract Link'),
        description=_(u'Link to the abstract of the work'),
        required=False)

CitationSchema = ATContentTypeSchema.copy() + Schema((

    StringField('cite_author',
        widget=StringWidget(label=_(u'Author(s)'),
                                  description=_(u'The author or authors of the work being cited'),
                                  label_msgid='label_author',
                                  i18n_domain='collective.types.citation'),
        required=False,
        searchable=True,
        storage=AnnotationStorage()),

    StringField('year',
        widget=StringWidget(label=_(u'Year'),
                            description=_(u'Publication Year'),
                            label_msgid='label_year',
                            i18n_domain='collective.types.citation'),
        required=False,
        searchable=True,
        storage=AnnotationStorage()),

    StringField('location',
        widget=StringWidget(label=_(u'Publication Location'),
                            description=_(u'Location of the publication office'),
                            label_msgid='label_location',
                            i18n_domain='collective.types.citation'),
        required=False,
        searchable=True,
        storage=AnnotationStorage()),

    StringField('fulltext_link',
        widget = StringWidget(label=_(u'Full Text URL'),
                                 description=_(u'Link to the full text of the work being cited'),
                                 label_msgid='label_full_text_link',
                                 i18n_domain='collective.types.citation'),
        required=False,
        searchable=False,
        storage=AnnotationStorage()),

    StringField('source',
        widget=StringWidget(label=_('Source/Journal'),
                            description=_('Source/Journal'),
                            label_msgid='label_source',
                            i18n_domain='collective.types.citation'),
        required=False,
        searchable=True,
        storage=AnnotationStorage()),

    StringField('volume',
        widget=StringWidget(label=_(u'Volume'),
                              description=_(u'Publication Volume'),
                              label_msgid='label_volume',
                              i18n_domain='collective.types.citation'),
        required=False,
        Searchable=False,
        storage=AnnotationStorage()),

    StringField('issue', 
        widget=StringWidget(label=_(u'Issue'),
                            description=_(u'Publication Issue'),
                            label_msgid='lable_issue',
                            i18n_domain='collective.types.citation'),
        required=False,
        searchable=False,
        storage=AnnotationStorage()),

    StringField('pages', 
        widget=IntegerWidget(label=_(u'Pages'),
                                  description=_(u'Number of pages'),
                                  label_msgid='label_pages',
                                  i18n_domain='collective.types.citation'),
        required=False,
        searchable=False,
        storage=AnnotationStorage()),

    StringField('abstract_link',
        widget=StringWidget(label=_(u'Abstract URL'),
                                     description=_(u'Link to the abstract of the work')),
        required=False,
        searchable=False,
        storage=AnnotationStorage()),
    ))

#Dublin core stuff
CitationSchema['title'].storage = AnnotationStorage()
CitationSchema['title'].widget.label = _(u"Title")
CitationSchema['title'].widget.description = _(u"")

CitationSchema['description'].storage = AnnotationStorage()
CitationSchema['description'].widget.label = _(u"Description")
CitationSchema['description'].widget.description = _("")

class Citation(base.ATCTContent):
    """Citation type for citing journals, publications,
    and links

    Make sure Citation implements ICitation
     >>> from collective.types.citation import ICitation
     >>> from collective.types.citation import Citation
     >>> from zope.interface.verify import verifyObject
     >>> cite = Citation('cite')
     >>> verifyObject(ICitation, cite)
     True
     
    """
    implements(ICitation)
    schema = CitationSchema
    portal_type = meta_type = 'collective.types.Citation'
    _at_rename_after_creation = True

    description = ATFieldProperty('description')
    cite_author = ATFieldProperty('cite_author')
    year = ATFieldProperty('year')
    location = ATFieldProperty('location')
    fulltext_link = ATFieldProperty('fulltext_link')
    source = ATFieldProperty('source')
    volume = ATFieldProperty('volume')
    issue = ATFieldProperty('issue')
    pages = ATFieldProperty('pages')
    abstract_link = ATFieldProperty('abstract_link')


base.registerATCT(Citation, 'collective.types.citation')
